@@ -34,7 +34,6 @@
 #include <string.h>
 #include <unistd.h>
 #include <ctype.h>
-#include <unistd.h>
 #include <fcntl.h>
 #include <time.h>
 #include <sys/time.h>
@@ -45,6 +44,7 @@
 #include "uuid.h"
 #include "uuid_vers.h"
 #include "uuid_md5.h"
+#include "uuid_sha1.h"
 #include "uuid_prng.h"
 #include "uuid_mac.h"
 #include "uuid_ui64.h"
@@ -83,6 +83,7 @@ struct uuid_st {
     uuid_obj_t     obj;                       /* inlined UUID object */
     prng_t        *prng;                      /* RPNG sub-object */
     md5_t         *md5;                       /* MD5 sub-object */
+    sha1_t        *sha1;                      /* SHA-1 sub-object */
     uuid_uint8_t   mac[IEEE_MAC_OCTETS];      /* pre-determined MAC address */
     struct timeval time_last;                 /* last retrieved timestamp */
     unsigned long  time_seq;                  /* last timestamp sequence counter */
@@ -107,6 +108,8 @@ uuid_rc_t uuid_create(uuid_t **uuid)
         return UUID_RC_INT;
     if (md5_create(&(*uuid)->md5) != MD5_RC_OK)
         return UUID_RC_INT;
+    if (sha1_create(&(*uuid)->sha1) != SHA1_RC_OK)
+        return UUID_RC_INT;
 
     /* resolve MAC address for insertion into node field of UUIDs */
     if (!mac_address((unsigned char *)((*uuid)->mac), sizeof((*uuid)->mac))) {
@@ -129,9 +132,10 @@ uuid_rc_t uuid_destroy(uuid_t *uuid)
     if (uuid == NULL)
         return UUID_RC_ARG;
 
-    /* destroy PRNG and MD5 sub-objects */
+    /* destroy PRNG, MD5 and SHA-1 sub-objects */
     prng_destroy(uuid->prng);
     md5_destroy(uuid->md5);
+    sha1_destroy(uuid->sha1);
 
     /* free UUID object */
     free(uuid);
@@ -140,7 +144,7 @@ uuid_rc_t uuid_destroy(uuid_t *uuid)
 }
 
 /* check whether UUID object represents "Nil UUID" */
-uuid_rc_t uuid_isnil(uuid_t *uuid, int *result)
+uuid_rc_t uuid_isnil(const uuid_t *uuid, int *result)
 {
     const unsigned char *ucp;
     int i;
@@ -162,7 +166,7 @@ uuid_rc_t uuid_isnil(uuid_t *uuid, int *result)
 }
 
 /* compare UUID objects */
-uuid_rc_t uuid_compare(uuid_t *uuid1, uuid_t *uuid2, int *result)
+uuid_rc_t uuid_compare(const uuid_t *uuid1, const uuid_t *uuid2, int *result)
 {
     int r;
 
@@ -171,11 +175,13 @@ uuid_rc_t uuid_compare(uuid_t *uuid1, uuid_t *uuid2, int *result)
         return UUID_RC_ARG;
 
     /* convenience macro for setting result */
-#   define RESULT(r) \
+#define RESULT(r) \
+    /*lint -save -e801 -e717*/ \
     do { \
         *result = (r); \
         goto result_exit; \
-    } while (0)
+    } while (0) \
+    /*lint -restore*/
 
     /* special cases: NULL or equal UUIDs */
     if (uuid1 == uuid2)
@@ -183,9 +189,9 @@ uuid_rc_t uuid_compare(uuid_t *uuid1, uuid_t *uuid2, int *result)
     if (uuid1 == NULL && uuid2 == NULL)
         RESULT(0);
     if (uuid1 == NULL)
-        RESULT((uuid_isnil(uuid2, &r), r) ? 0 : -1);
+        RESULT((uuid_isnil(uuid2, &r) == UUID_RC_OK ? r : 0) ? 0 : -1);
     if (uuid2 == NULL)
-        RESULT((uuid_isnil(uuid1, &r), r) ? 0 : 1);
+        RESULT((uuid_isnil(uuid1, &r) == UUID_RC_OK ? r : 0) ? 0 : 1);
 
     /* standard cases: regular different UUIDs */
     if (uuid1->obj.time_low != uuid2->obj.time_low)
@@ -237,12 +243,12 @@ static uuid_rc_t uuid_import_bin(uuid_t *uuid, const void *data_ptr, size_t data
 
     /* unpack "time_mid" field */
     tmp16 = *in++;
-    tmp16 = (tmp16 << 8) | *in++;
+    tmp16 = (uuid_uint16_t)(tmp16 << 8) | *in++;
     uuid->obj.time_mid = tmp16;
 
     /* unpack "time_hi_and_version" field */
     tmp16 = *in++;
-    tmp16 = (tmp16 << 8) | *in++;
+    tmp16 = (uuid_uint16_t)(tmp16 << 8) | *in++;
     uuid->obj.time_hi_and_version = tmp16;
 
     /* unpack "clock_seq_hi_and_reserved" field */
@@ -260,7 +266,7 @@ static uuid_rc_t uuid_import_bin(uuid_t *uuid, const void *data_ptr, size_t data
 
 /* INTERNAL: pack UUID object into binary representation
    (allows in-place operation for internal efficiency!) */
-static uuid_rc_t uuid_export_bin(uuid_t *uuid, void **data_ptr, size_t *data_len)
+static uuid_rc_t uuid_export_bin(const uuid_t *uuid, void **data_ptr, size_t *data_len)
 {
     uuid_uint8_t *out;
     uuid_uint32_t tmp32;
@@ -382,27 +388,30 @@ static uuid_rc_t uuid_import_str(uuid_t *uuid, const void *data_ptr, size_t data
     for (i = 0; i < sizeof(uuid->obj.node); i++) {
         hexbuf[0] = *cp++;
         hexbuf[1] = *cp++;
-        uuid->obj.node[i] = strtoul(hexbuf, NULL, 16);
+        uuid->obj.node[i] = (uuid_uint8_t)strtoul(hexbuf, NULL, 16);
     }
 
     return UUID_RC_OK;
 }
 
 /* INTERNAL: export UUID object to string representation */
-static uuid_rc_t uuid_export_str(uuid_t *uuid, void **data_ptr, size_t *data_len)
+static uuid_rc_t uuid_export_str(const uuid_t *uuid, void **data_ptr, size_t *data_len)
 {
+    char *data_buf;
+
     /* sanity check argument(s) */
     if (uuid == NULL || data_ptr == NULL)
         return UUID_RC_ARG;
 
-    /* allocate output buffer */
+    /* determine output buffer */
     if (*data_ptr == NULL) {
-        if ((*data_ptr = (void *)malloc(UUID_LEN_STR+1)) == NULL)
+        if ((data_buf = (void *)malloc(UUID_LEN_STR+1)) == NULL)
             return UUID_RC_MEM;
         if (data_len != NULL)
             *data_len = UUID_LEN_STR+1;
     }
     else {
+        data_buf = (char *)(*data_ptr);
         if (data_len == NULL)
             return UUID_RC_ARG;
         if (*data_len < UUID_LEN_STR+1)
@@ -411,7 +420,7 @@ static uuid_rc_t uuid_export_str(uuid_t *uuid, void **data_ptr, size_t *data_len
     }
 
     /* format UUID into string representation */
-    str_snprintf((char *)(*data_ptr), UUID_LEN_STR+1,
+    if (str_snprintf(data_buf, UUID_LEN_STR+1,
         "%08lx-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x",
         (unsigned long)uuid->obj.time_low,
         (unsigned int)uuid->obj.time_mid,
@@ -423,7 +432,15 @@ static uuid_rc_t uuid_export_str(uuid_t *uuid, void **data_ptr, size_t *data_len
         (unsigned int)uuid->obj.node[2],
         (unsigned int)uuid->obj.node[3],
         (unsigned int)uuid->obj.node[4],
-        (unsigned int)uuid->obj.node[5]);
+        (unsigned int)uuid->obj.node[5]) != UUID_LEN_STR) {
+        if (*data_ptr == NULL)
+            free(data_buf);
+        return UUID_RC_INT;
+    }
+
+    /* pass back new buffer if locally allocated */
+    if (*data_ptr == NULL)
+        *data_ptr = data_buf;
 
     return UUID_RC_OK;
 }
@@ -443,12 +460,13 @@ static struct {
     const char *desc;
 } uuid_dectab_version[] = {
     { 1, "time and node based" },
-    { 3, "name based" },
-    { 4, "random data based" }
+    { 3, "name based, MD5" },
+    { 4, "random data based" },
+    { 5, "name based, SHA-1" }
 };
 
 /* INTERNAL: dump UUID object as descriptive text */
-static uuid_rc_t uuid_export_txt(uuid_t *uuid, void **data_ptr, size_t *data_len)
+static uuid_rc_t uuid_export_txt(const uuid_t *uuid, void **data_ptr, size_t *data_len)
 {
     uuid_rc_t rc;
     char **out;
@@ -569,13 +587,15 @@ static uuid_rc_t uuid_export_txt(uuid_t *uuid, void **data_ptr, size_t *data_len
         /* decode anything else as hexadecimal byte-string only */
 
         /* determine annotational hint */
-        content = "not decipherable, because unknown UUID version";
+        content = "not decipherable: unknown UUID version";
         if (isnil)
-            content = "special case of DCE 1.1 Nil UUID";
+            content = "special case: DCE 1.1 Nil UUID";
         else if (tmp16 == 3)
-            content = "not decipherable, because message digest only";
+            content = "not decipherable: MD5 message digest only";
         else if (tmp16 == 4)
-            content = "no semantics, because random data only";
+            content = "no semantics: random data only";
+        else if (tmp16 == 5)
+            content = "not decipherable: truncated SHA-1 message digest only";
 
         /* pack UUID into binary representation */
         tmp_ptr = (void *)&tmp_bin;
@@ -638,7 +658,7 @@ uuid_rc_t uuid_import(uuid_t *uuid, uuid_fmt_t fmt, const void *data_ptr, size_t
 }
 
 /* UUID exporting */
-uuid_rc_t uuid_export(uuid_t *uuid, uuid_fmt_t fmt, void **data_ptr, size_t *data_len)
+uuid_rc_t uuid_export(const uuid_t *uuid, uuid_fmt_t fmt, void **data_ptr, size_t *data_len)
 {
     uuid_rc_t rc;
 
@@ -658,11 +678,11 @@ uuid_rc_t uuid_export(uuid_t *uuid, uuid_fmt_t fmt, void **data_ptr, size_t *dat
 }
 
 /* INTERNAL: brand UUID with version and variant */
-static void uuid_brand(uuid_t *uuid, int version)
+static void uuid_brand(uuid_t *uuid, unsigned int version)
 {
     /* set version (as given) */
     uuid->obj.time_hi_and_version &= BM_MASK(11,0);
-    uuid->obj.time_hi_and_version |= BM_SHL((uuid_uint16_t)version, 12);
+    uuid->obj.time_hi_and_version |= (uuid_uint16_t)BM_SHL(version, 12);
 
     /* set variant (always DCE 1.1 only) */
     uuid->obj.clock_seq_hi_and_reserved &= BM_MASK(5,0);
@@ -689,7 +709,7 @@ static uuid_rc_t uuid_make_v1(uuid_t *uuid, unsigned int mode, va_list ap)
      */
 
     /* determine current system time and sequence counter */
-    while (1) {
+    for (;;) {
         /* determine current system time */
         if (gettimeofday(&time_now, NULL) == -1)
             return UUID_RC_SYS;
@@ -725,9 +745,9 @@ static uuid_rc_t uuid_make_v1(uuid_t *uuid, unsigned int mode, va_list ap)
     }
 
     /* convert from timeval (sec,usec) to OSSP ui64 (100*nsec) format */
-    t = ui64_n2i(time_now.tv_sec);
+    t = ui64_n2i((unsigned long)time_now.tv_sec);
     t = ui64_muln(t, 1000000, NULL);
-    t = ui64_addn(t, time_now.tv_usec, NULL);
+    t = ui64_addn(t, (int)time_now.tv_usec, NULL);
     t = ui64_muln(t, 10, NULL);
 
     /* adjust for offset between UUID and Unix Epoch time */
@@ -737,7 +757,7 @@ static uuid_rc_t uuid_make_v1(uuid_t *uuid, unsigned int mode, va_list ap)
     /* compensate for low resolution system clock by adding
        the time/tick sequence counter */
     if (uuid->time_seq > 0)
-        t = ui64_addn(t, uuid->time_seq, NULL);
+        t = ui64_addn(t, (int)uuid->time_seq, NULL);
 
     /* store the 60 LSB of the time in the UUID */
     t = ui64_rol(t, 16, &ov);
@@ -782,7 +802,8 @@ static uuid_rc_t uuid_make_v1(uuid_t *uuid, unsigned int mode, va_list ap)
 
     if ((mode & UUID_MAKE_MC) || (uuid->mac[0] & BM_OCTET(1,0,0,0,0,0,0,0))) {
         /* generate random IEEE 802 local multicast MAC address */
-        prng_data(uuid->prng, (void *)&(uuid->obj.node), sizeof(uuid->obj.node));
+        if (prng_data(uuid->prng, (void *)&(uuid->obj.node), sizeof(uuid->obj.node)) != PRNG_RC_OK)
+            return UUID_RC_INT;
         uuid->obj.node[0] |= IEEE_MAC_MCBIT;
         uuid->obj.node[0] |= IEEE_MAC_LOBIT;
     }
@@ -852,7 +873,7 @@ uuid_rc_t uuid_load(uuid_t *uuid, const char *name)
     return UUID_RC_OK;
 }
 
-/* INTERNAL: generate UUID version 3: name based */
+/* INTERNAL: generate UUID version 3: name based with MD5 */
 static uuid_rc_t uuid_make_v3(uuid_t *uuid, unsigned int mode, va_list ap)
 {
     char *str;
@@ -909,6 +930,56 @@ static uuid_rc_t uuid_make_v4(uuid_t *uuid, unsigned int mode, va_list ap)
     return UUID_RC_OK;
 }
 
+/* INTERNAL: generate UUID version 5: name based with SHA-1 */
+static uuid_rc_t uuid_make_v5(uuid_t *uuid, unsigned int mode, va_list ap)
+{
+    char *str;
+    uuid_t *uuid_ns;
+    uuid_uint8_t uuid_buf[UUID_LEN_BIN];
+    void *uuid_ptr;
+    size_t uuid_len;
+    uuid_uint8_t sha1_buf[SHA1_LEN_BIN];
+    void *sha1_ptr;
+
+    /* determine namespace UUID and name string arguments */
+    if ((uuid_ns = (uuid_t *)va_arg(ap, void *)) == NULL)
+        return UUID_RC_ARG;
+    if ((str = (char *)va_arg(ap, char *)) == NULL)
+        return UUID_RC_ARG;
+
+    /* initialize SHA-1 context */
+    if (sha1_init(uuid->sha1) != SHA1_RC_OK)
+        return UUID_RC_MEM;
+
+    /* load the namespace UUID into SHA-1 context */
+    uuid_ptr = (void *)&uuid_buf;
+    uuid_len = sizeof(uuid_buf);
+    uuid_export(uuid_ns, UUID_FMT_BIN, &uuid_ptr, &uuid_len);
+    sha1_update(uuid->sha1, uuid_buf, uuid_len);
+
+    /* load the argument name string into SHA-1 context */
+    sha1_update(uuid->sha1, str, strlen(str));
+
+    /* store SHA-1 result into UUID
+       (requires SHA1_LEN_BIN space, but UUID_LEN_BIN space is available
+       only, so use a temporary buffer to store SHA-1 results and then
+       use lower part only according to standard */
+    sha1_ptr = (void *)sha1_buf;
+    sha1_store(uuid->sha1, &sha1_ptr, NULL);
+    uuid_ptr = (void *)&(uuid->obj);
+    memcpy(uuid_ptr, sha1_ptr, UUID_LEN_BIN);
+
+    /* fulfill requirement of standard and convert UUID data into
+       local/host byte order (this uses fact that uuid_import_bin() is
+       able to operate in-place!) */
+    uuid_import(uuid, UUID_FMT_BIN, (void *)&(uuid->obj), UUID_LEN_BIN);
+
+    /* brand UUID with version and variant */
+    uuid_brand(uuid, 5);
+
+    return UUID_RC_OK;
+}
+
 /* generate UUID */
 uuid_rc_t uuid_make(uuid_t *uuid, unsigned int mode, ...)
 {
@@ -927,6 +998,8 @@ uuid_rc_t uuid_make(uuid_t *uuid, unsigned int mode, ...)
         rc = uuid_make_v3(uuid, mode, ap);
     else if (mode & UUID_MAKE_V4)
         rc = uuid_make_v4(uuid, mode, ap);
+    else if (mode & UUID_MAKE_V5)
+        rc = uuid_make_v5(uuid, mode, ap);
     else
         rc = UUID_RC_ARG;
     va_end(ap);
