@@ -1,7 +1,7 @@
 /*
 **  OSSP uuid - Universally Unique Identifier
-**  Copyright (c) 2004 Ralf S. Engelschall <rse@engelschall.com>
-**  Copyright (c) 2004 The OSSP Project <http://www.ossp.org/>
+**  Copyright (c) 2004-2005 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 2004-2005 The OSSP Project <http://www.ossp.org/>
 **
 **  This file is part of OSSP uuid, a library for the generation
 **  of UUIDs which can found at http://www.ossp.org/pkg/lib/uuid/
@@ -100,7 +100,7 @@ static void dopr_outch (char *, size_t *, size_t, int);
 
 /* some handy macros */
 #define char_to_int(p) (p - '0')
-#define MAX(p,q) ((p >= q) ? p : q)
+#define STR_MAX(p,q) ((p >= q) ? p : q)
 #define NUL '\0'
 
 static void
@@ -281,6 +281,7 @@ dopr(
                 break;
             case 'E':
                 flags |= DP_F_UP;
+                /* FALLTHROUGH */
             case 'e':
                 if (cflags == DP_C_LDOUBLE)
                     fvalue = va_arg(args, LDOUBLE);
@@ -289,6 +290,7 @@ dopr(
                 break;
             case 'G':
                 flags |= DP_F_UP;
+                /* FALLTHROUGH */
             case 'g':
                 if (cflags == DP_C_LDOUBLE)
                     fvalue = va_arg(args, LDOUBLE);
@@ -322,7 +324,7 @@ dopr(
                     num = va_arg(args, LLONG *);
                     *num = (LLONG) currlen;
                 } else {
-                    int    *num;
+                    int *num;
                     num = va_arg(args, int *);
                     *num = currlen;
                 }
@@ -440,13 +442,13 @@ fmtint(
     convert[place] = 0;
 
     zpadlen = max - place;
-    spadlen = min - MAX(max, place) - (signvalue ? 1 : 0);
+    spadlen = min - STR_MAX(max, place) - (signvalue ? 1 : 0);
     if (zpadlen < 0)
         zpadlen = 0;
     if (spadlen < 0)
         spadlen = 0;
     if (flags & DP_F_ZERO) {
-        zpadlen = MAX(zpadlen, spadlen);
+        zpadlen = STR_MAX(zpadlen, spadlen);
         spadlen = 0;
     }
     if (flags & DP_F_MINUS)
@@ -482,7 +484,7 @@ fmtint(
 }
 
 static LDOUBLE
-abs_val(LDOUBLE value)
+math_abs(LDOUBLE value)
 {
     LDOUBLE result = value;
     if (value < 0)
@@ -491,7 +493,7 @@ abs_val(LDOUBLE value)
 }
 
 static LDOUBLE
-pow10(int exponent)
+math_pow10(int exponent)
 {
     LDOUBLE result = 1;
     while (exponent > 0) {
@@ -502,7 +504,7 @@ pow10(int exponent)
 }
 
 static long
-round(LDOUBLE value)
+math_round(LDOUBLE value)
 {
     long intpart;
     intpart = (long) value;
@@ -536,7 +538,7 @@ fmtfp(
 
     if (max < 0)
         max = 6;
-    ufvalue = abs_val(fvalue);
+    ufvalue = math_abs(fvalue);
     if (fvalue < 0)
         signvalue = '-';
     else if (flags & DP_F_PLUS)
@@ -553,11 +555,11 @@ fmtfp(
 
     /* we "cheat" by converting the fractional part to integer by
        multiplying by a factor of 10 */
-    fracpart = round((pow10(max)) * (ufvalue - intpart));
+    fracpart = math_round((math_pow10(max)) * (ufvalue - intpart));
 
-    if (fracpart >= pow10(max)) {
+    if (fracpart >= math_pow10(max)) {
         intpart++;
-        fracpart -= (long)pow10(max);
+        fracpart -= (long)math_pow10(max);
     }
 
     /* convert integer part */
